@@ -19,7 +19,7 @@
  */
 
 #include "config.h"
-#include <libgnome/libgnome.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <X11/Xauth.h>
@@ -35,10 +35,11 @@
 #include <sys/un.h>
 #include <errno.h>
 
-#include <viciousui.h>
+#include "vicious.h"
 
 #include "gdm.h"
 #include "gdmcomm.h"
+#include "gdmconfig.h"
 
 static gboolean debug = FALSE;
 
@@ -110,44 +111,59 @@ version_ok_p (const char *version, const char *min_version)
 char *
 gdmcomm_call_gdm (const char *command, const char * auth_cookie, const char *min_version, int tries)
 {
-	struct sockaddr_un addr;
-	int fd;
+	static int fd = 0;
+	static int num_cmds = 0;
 	char *ret;
 
+	if (num_cmds == (GDM_SUP_MAX_CONNECTIONS - 1)) {
+	   do_command (fd, GDM_SUP_CLOSE, FALSE);
+	   VE_IGNORE_EINTR (close (fd));
+	   fd       = 0;
+	   num_cmds = 0;
+	}
+
+	num_cmds++;
+
 	if (tries <= 0)
 		return NULL;
 
-	fd = socket (AF_UNIX, SOCK_STREAM, 0);
-	if (fd < 0) {
-		return gdmcomm_call_gdm (command, auth_cookie, min_version, tries - 1);
-	}
+	if (fd <= 0) {
+		struct sockaddr_un addr;
+		strcpy (addr.sun_path, GDM_SUP_SOCKET);
+		addr.sun_family = AF_UNIX;
 
-	strcpy (addr.sun_path, GDM_SUP_SOCKET);
-	addr.sun_family = AF_UNIX;
+		fd = socket (AF_UNIX, SOCK_STREAM, 0);
+		if (fd < 0) {
+			return gdmcomm_call_gdm (command, auth_cookie, min_version, tries - 1);
+		}
 
-	if (connect (fd, (struct sockaddr *)&addr, sizeof (addr)) < 0) {
-		VE_IGNORE_EINTR (close (fd));
-		return gdmcomm_call_gdm (command, auth_cookie, min_version, tries - 1);
-	}
+		if (connect (fd, (struct sockaddr *)&addr, sizeof (addr)) < 0) {
+			VE_IGNORE_EINTR (close (fd));
+			fd = 0;
+			return gdmcomm_call_gdm (command, auth_cookie, min_version, tries - 1);
+		}
 
-	/* Version check first */
-	ret = do_command (fd, GDM_SUP_VERSION, TRUE /* get_response */);
-	if (ret == NULL) {
-		VE_IGNORE_EINTR (close (fd));
-		return gdmcomm_call_gdm (command, auth_cookie, min_version, tries - 1);
-	}
-	if (strncmp (ret, "GDM ", strlen ("GDM ")) != 0) {
+		/* Version check first - only check first time */
+		ret = do_command (fd, GDM_SUP_VERSION, TRUE /* get_response */);
+		if (ret == NULL) {
+			VE_IGNORE_EINTR (close (fd));
+			fd = 0;
+			return gdmcomm_call_gdm (command, auth_cookie, min_version, tries - 1);
+		}
+		if (strncmp (ret, "GDM ", strlen ("GDM ")) != 0) {
+			g_free (ret);
+			VE_IGNORE_EINTR (close (fd));
+			fd = 0;
+			return NULL;
+		}
+		if ( ! version_ok_p (&ret[4], min_version)) {
+			g_free (ret);
+			VE_IGNORE_EINTR (close (fd));
+			fd = 0;
+			return NULL;
+		}
 		g_free (ret);
-		VE_IGNORE_EINTR (close (fd));
-		return NULL;
 	}
-	if ( ! version_ok_p (&ret[4], min_version)) {
-		g_free (ret);
-		do_command (fd, GDM_SUP_CLOSE, FALSE /* get_response */);
-		VE_IGNORE_EINTR (close (fd));
-		return NULL;
-	}
-	g_free (ret);
 
 	/* require authentication */
 	if (auth_cookie != NULL)  {
@@ -157,14 +173,14 @@ gdmcomm_call_gdm (const char *command, const char * auth_cookie, const char *min
 		g_free (auth_cmd);
 		if (ret == NULL) {
 			VE_IGNORE_EINTR (close (fd));
+			fd = 0;
 			return gdmcomm_call_gdm (command, auth_cookie,
 						 min_version, tries - 1);
 		}
 		/* not auth'ed */
 		if (strcmp (ret, "OK") != 0) {
-			do_command (fd, GDM_SUP_CLOSE,
-				    FALSE /* get_response */);
 			VE_IGNORE_EINTR (close (fd));
+			fd = 0;
 			/* returns the error */
 			return ret;
 		}
@@ -174,13 +190,10 @@ gdmcomm_call_gdm (const char *command, const char * auth_cookie, const char *min
 	ret = do_command (fd, command, TRUE /* get_response */);
 	if (ret == NULL) {
 		VE_IGNORE_EINTR (close (fd));
+		fd = 0;
 		return gdmcomm_call_gdm (command, auth_cookie, min_version, tries - 1);
 	}
 
-	do_command (fd, GDM_SUP_CLOSE, FALSE /* get_response */);
-
-	VE_IGNORE_EINTR (close (fd));
-
 	return ret;
 }
 
@@ -368,7 +381,7 @@ gdmcomm_get_auth_cookie (void)
 }
 
 gboolean
-gdmcomm_check (gchar *config_file, gboolean gui_bitching)
+gdmcomm_check (gboolean show_dialog)
 {
 	GtkWidget *dialog;
 	FILE *fp = NULL;
@@ -377,11 +390,7 @@ gdmcomm_check (gchar *config_file, gboolean gui_bitching)
 	struct stat s;
 	int statret;
 
-	if (config_file == NULL)
-		return FALSE;
-
-	pidfile = ve_config_get_string (ve_config_get (config_file),
-					GDM_KEY_PID_FILE);
+	pidfile = gdm_config_get_string (GDM_KEY_PID_FILE);
 
 	pid = 0;
 	if (pidfile != NULL)
@@ -394,13 +403,11 @@ gdmcomm_check (gchar *config_file, gboolean gui_bitching)
 			pid = 0;
 	}
 
-	g_free (pidfile);
-
 	errno = 0;
 	if (pid <= 1 ||
 	    (kill (pid, 0) < 0 &&
 	     errno != EPERM)) {
-		if (gui_bitching) {
+		if (show_dialog) {
 			dialog = ve_hig_dialog_new
 				(NULL /* parent */,
 				 GTK_DIALOG_MODAL /* flags */,
@@ -422,11 +429,11 @@ gdmcomm_check (gchar *config_file, gboolean gui_bitching)
 		return FALSE;
 	}
 
-	VE_IGNORE_EINTR (statret = stat (GDM_SUP_SOCKET, &s));
+	VE_IGNORE_EINTR (statret = g_stat (GDM_SUP_SOCKET, &s));
 	if (statret < 0 ||
 	    s.st_uid != 0 ||
-	    access (GDM_SUP_SOCKET, R_OK|W_OK) != 0) {
-		if (gui_bitching) {
+	    g_access (GDM_SUP_SOCKET, R_OK|W_OK) != 0) {
+		if (show_dialog) {
 			dialog = ve_hig_dialog_new
 				(NULL /* parent */,
 				 GTK_DIALOG_MODAL /* flags */,
@@ -451,10 +458,10 @@ gdmcomm_get_error_message (const char *ret, gboolean use_xnest)
 {
 	/* These need a bit more refinement */
 	if (ret == NULL) {
-		return _("Cannot communicate with gdm. Perhaps "
+		return _("Cannot communicate with GDM. Perhaps "
 			 "you have an old version running.");
 	} else if (strncmp (ret, "ERROR 0 ", strlen ("ERROR 0 ")) == 0) {
-		return _("Cannot communicate with gdm. Perhaps "
+		return _("Cannot communicate with GDM. Perhaps "
 			 "you have an old version running.");
 	} else if (strncmp (ret, "ERROR 1 ", strlen ("ERROR 1 ")) == 0) {
 		return _("The allowed limit of flexible X servers reached.");
@@ -472,7 +479,7 @@ gdmcomm_get_error_message (const char *ret, gboolean use_xnest)
 	} else if (strncmp (ret, "ERROR 6 ", strlen ("ERROR 6 ")) == 0) {
 		if (use_xnest)
 			return _("The nested X server (Xnest) is not "
-				 "available, or gdm is badly configured.\n"
+				 "available, or GDM is badly configured.\n"
 				 "Please install the Xnest package in "
 				 "order to use the nested login.");
 		else
@@ -492,7 +499,7 @@ gdmcomm_get_error_message (const char *ret, gboolean use_xnest)
 			 "for this operation.  Perhaps your .Xauthority "
 			 "file is not set up correctly.");
 	} else if (strncmp (ret, "ERROR 200 ", strlen ("ERROR 200 ")) == 0) {
-		return _("Too many messages were sent to gdm and it hung up"
+		return _("Too many messages were sent to GDM and it hung up "
 			 "on us.");
 	} else {
 		return _("Unknown error occurred.");
