@@ -36,6 +36,7 @@
 static GHashTable *int_hash       = NULL;
 static GHashTable *bool_hash      = NULL;
 static GHashTable *string_hash    = NULL;
+static gboolean gdm_never_cache   = FALSE;
 
 /*
  * Hack to keep track if config functions should be printing error messages
@@ -50,6 +51,24 @@ gdm_openlog (const char *ident, int logopt, int facility)
    using_syslog = TRUE;
 }
 
+/**
+ * gdm_config_never_cache
+ *
+ * Most programs want config data to be cached to avoid constantly
+ * grabbing the information over the wire and are happy calling 
+ * gdm_update_config to update a key value.  However, gdmsetup
+ * really does want the latest value each time it accesses a 
+ * config option.  To avoid needing to call update_config 
+ * for each key to be retrieved, just calling this function will
+ * let the config system know to always get the value via the
+ * sockets connection.
+ */
+void
+gdm_config_never_cache (gboolean never_cache)
+{
+   gdm_never_cache = never_cache;
+}
+
 /**
  * gdm_config_hash_lookup
  *
@@ -116,10 +135,131 @@ gdm_config_get_result (gchar *key)
 	result = gdmcomm_call_gdm (command, NULL /* auth cookie */,
 		"2.13.0.1", 5);
 
+	g_free (command);
 	g_free (newkey);
 	return result;
 }
 
+
+/**
+ * gdm_config_get_xserver_details
+ *
+ * Calls daemon to get details for an xserver config.
+ */
+static gchar *
+gdm_config_get_xserver_details (gchar *xserver, gchar *key)
+{
+	gchar *command = NULL;
+	gchar *result  = NULL;
+	gchar *temp;
+
+	command = g_strdup_printf ("GET_SERVER_DETAILS %s %s", xserver, key);
+	result = gdmcomm_call_gdm (command, NULL /* auth cookie */,
+		"2.13.0.1", 5);
+
+	g_free (command);
+
+	if (! result || ve_string_empty (result) ||
+	    strncmp (result, "OK ", 3) != 0) {
+
+		if (using_syslog)
+			syslog (LOG_ERR, "Could not access xserver configuration");
+		else
+			printf ("Could not access xserver configuration");
+
+		if (result)
+			g_free (result);
+		return NULL;
+	}
+
+	/* skip the "OK " */
+	temp = g_strdup (result + 3);
+	g_free (result);
+
+	return temp;
+}
+
+/**
+ * gdm_config_get_xservers
+ *
+ * Calls daemon to get xserver config.
+ */
+GSList *
+gdm_config_get_xservers (gboolean flexible)
+{
+	GSList *xservers = NULL;
+	gchar *p;
+        gchar **splitstr, **sec;
+	gchar *command = NULL;
+	gchar *result  = NULL;
+	gchar *temp;
+	gboolean tempbool;
+
+	command = g_strdup_printf ("GET_SERVER_LIST");
+	result = gdmcomm_call_gdm (command, NULL /* auth cookie */,
+		"2.13.0.1", 5);
+
+	g_free (command);
+
+	if (! result || ve_string_empty (result) ||
+	    strncmp (result, "OK ", 3) != 0) {
+
+		if (using_syslog)
+			syslog (LOG_ERR, "Could not access xserver configuration");
+		else
+			printf ("Could not access xserver configuration");
+
+		if (result)
+			g_free (result);
+		return NULL;
+	}
+
+	/* skip the "OK " */
+        splitstr = g_strsplit (result + 3, ";", 0);
+	sec = splitstr;
+	g_free (result);
+
+        while (*sec != NULL) {
+		GdmXserver *svr = g_new0 (GdmXserver, 1);
+
+		svr->id      = gdm_config_get_xserver_details (*sec, "ID");
+		svr->name    = gdm_config_get_xserver_details (*sec, "NAME");
+		svr->command = gdm_config_get_xserver_details (*sec, "COMMAND");
+
+		temp = gdm_config_get_xserver_details (*sec, "FLEXIBLE");
+		if (g_strncasecmp (temp, "true", 4) == 0)
+			svr->flexible = TRUE;
+		else
+			svr->flexible = FALSE;
+		temp = gdm_config_get_xserver_details (*sec, "CHOOSABLE");
+		if (g_strncasecmp (temp, "true", 4) == 0)
+			svr->choosable = TRUE;
+		else
+			svr->choosable = FALSE;
+		temp = gdm_config_get_xserver_details (*sec, "HANDLED");
+		if (g_strncasecmp (temp, "true", 4) == 0)
+			svr->handled = TRUE;
+		else
+			svr->handled = FALSE;
+		temp = gdm_config_get_xserver_details (*sec, "CHOOSER");
+		if (g_strncasecmp (temp, "true", 4) == 0)
+			svr->chooser = TRUE;
+		else
+			svr->chooser = FALSE;
+
+		sec++;
+
+		/* If only flexible was requested, then skip if not flexible */
+		if (flexible && !svr->flexible)
+			continue;
+
+		xservers = g_slist_append (xservers, svr);
+	}
+
+	g_strfreev (splitstr);
+	return xservers;
+}
+
 /**
  * gdm_config_get_string
  *
@@ -164,7 +304,7 @@ _gdm_config_get_string (gchar *key, gboolean reload, gboolean *changed, gboolean
 	g_free (result);
 
 	if (hashretval == NULL) {
-		gchar** charval = g_new0 (gchar *, 1);
+		gchar **charval = g_new0 (gchar *, 1);
 		*charval = temp;
 		gdm_config_add_hash (string_hash, key, charval);
 
@@ -188,7 +328,10 @@ _gdm_config_get_string (gchar *key, gboolean reload, gboolean *changed, gboolean
 gchar *
 gdm_config_get_string (gchar *key)
 {
-   return _gdm_config_get_string (key, FALSE, NULL, TRUE);
+   if (gdm_never_cache == TRUE)
+      return _gdm_config_get_string (key, TRUE, NULL, TRUE);
+   else
+      return _gdm_config_get_string (key, FALSE, NULL, TRUE);
 }
 
 /**
@@ -238,7 +381,10 @@ _gdm_config_get_translated_string (gchar *key, gboolean reload, gboolean *change
 gchar *
 gdm_config_get_translated_string (gchar *key)
 {
-   return _gdm_config_get_translated_string (key, FALSE, NULL);
+   if (gdm_never_cache == TRUE)
+      return _gdm_config_get_translated_string (key, TRUE, NULL);
+   else
+      return _gdm_config_get_translated_string (key, FALSE, NULL);
 }
 
 /**
@@ -306,7 +452,10 @@ _gdm_config_get_int (gchar *key, gboolean reload, gboolean *changed)
 gint
 gdm_config_get_int (gchar *key)
 {
-   return _gdm_config_get_int (key, FALSE, NULL);
+   if (gdm_never_cache == TRUE)
+      return _gdm_config_get_int (key, TRUE, NULL);
+   else
+      return _gdm_config_get_int (key, FALSE, NULL);
 }
 
 /**
@@ -377,7 +526,10 @@ _gdm_config_get_bool (gchar *key, gboolean reload, gboolean *changed)
 gboolean
 gdm_config_get_bool (gchar *key)
 {
-   return _gdm_config_get_bool (key, FALSE, NULL);
+   if (gdm_never_cache == TRUE)
+      return _gdm_config_get_bool (key, TRUE, NULL);
+   else
+      return _gdm_config_get_bool (key, FALSE, NULL);
 }
 
 /**
